/* eslint-disable no-undef */
/* eslint-disable quotes */
/* eslint-disable indent */
/**
 *
 * ApprovePost
 *
 */

import cn from 'classnames';
import React, { useEffect, useState } from 'react';
import { FormattedMessage } from 'react-intl';
import { auth, Button, LoadingIndicatorPage, PluginHeader } from 'strapi-helper-plugin';
// Design
import PageTitle from '../../components/PageTitle';
import styles from './styles.css';




async function handleApi(action, url, params) {
  const link = url;    
  const headersData = new Headers();
  headersData.append("Authorization", `Bearer ${auth.getToken()}`);

  const formData = new FormData();
  for (let key in params) {
      formData.append(key, params[key]);
  }

  const response = await fetch(link, {
    method: action,
    headers: headersData,
    body: action === 'POST' ? formData : undefined,
  });
  const data = await response.json();
  if (data.statusCode) {
    throw data;
  } else {
    return data;
  }
}
const ApprovePost = ({ isLoading, history }) => {

  const [posts, setPosts] = useState([]);
  const loadPosts = () => {
    handleApi('GET', "/posts/unapproved", {})
      .then(posts => {
        setPosts(posts);
      }).catch(callback => {
        console.log("ERROR");
        console.log(callback);
      });
  };
  useEffect(() => {
    loadPosts();
  }, []);

  const renderHelmet = message => <PageTitle title={message} />;
  // const onTest(test) {
  //   this.setState({
  //     test,
  //   })
  // }

  if (isLoading) {
    return <LoadingIndicatorPage />;
  }
  const renderMedia = mediaArray => {
    if (mediaArray && mediaArray.length && mediaArray[0].url) {
      const media = mediaArray[0];
      const type = media.mime.split('/')[0];
      const { url } = media;
      if (type === "video") {
        return <video style={{width: '100%'}} controls>
          <source src={url} type={media.mime} />
          No support for video tag
          </video>;
      } else if (type === "image") {
        return <a href={url} style={{
          width: '100%',
        }} target="_blank"><img src={url} style={{
          objectFit: "contain",
          width: '100%',
        }} />
        </a>;
      }
      return undefined;
    }
    return undefined;
  };
  const onApprove = (id) => {
    handleApi('GET', `/posts/approve/${id}`, {}).then(() => {
      loadPosts();
    }).catch(callback => {
      console.log("Error", callback);
    });
  };
  const onDelete = (id) => {

    handleApi('POST', `/posts/remove`, {
      id,
    }).then(() => {
      loadPosts();
    }).catch(callback => {
      console.log("Error", callback);
    });
  };

  const onPressPost = (id) => {
    history.push(`/plugins/content-manager/post/${id}?source=content-manager`);
  };
  return (
    <div>
      <FormattedMessage id="post.approval.helmet" defaultMessage="Post approval" >
        {renderHelmet}
      </FormattedMessage>
      <div className={cn('container-fluid', styles.containerFluid)}>

        <PluginHeader
          title="Post approval"
          description="Approve post for zingr app"
          actions={[]}
        />
        <div className="row" style={{
          paddingTop: "30px",
        }}>
          <div className="col-12">
            <table className={cn(styles.postListTable)}>
              <thead className={cn(styles.postListHead)}>
                {/* <th>Id</th> */}
                <th>About</th>
                {/* <th>Username</th> */}
                <th>Media</th>
                <th>Approve</th>
                <th>Delete</th>
              </thead>
              <tbody>
                {posts.map(post => {
                  return <tr className={cn(styles.postListEntry)}>
                    {/* <td>{post.id}</td> */}
                    <td onClick={() => onPressPost(post.id)}>{post.about}</td>
                    {/* <td>{post.user && post.user.username}</td> */}
                    <td>{renderMedia(post.media)}</td>
                    <td><Button label="Approve" kind="primaryAddShape" primary={true} primaryAddShape={true} onClick={(e) => {
                      e.preventDefault();
                      e.stopPropagation();
                      onApprove(post.id);
                    }} /></td>
                     <td><Button label="app.utils.delete" kind="delete" primary={true} primaryAddShape={true} onClick={(e) => {
                      e.preventDefault();
                      e.stopPropagation();
                      onDelete(post.id);
                    }} /></td>
                  </tr>;
                })}
              </tbody>
            </table>
          </div>
        </div>
        {/* <input type="text" onChange={e => this.onTest(e.target.value)} />
          <button value="" name="Ok" onClick={() => this.loadTest()} /> */}
      </div>
    </div>
  );
};


export default ApprovePost;
