'use strict';

/**
 * Read the documentation (https://strapi.io/documentation/3.0.0-beta.x/guides/controllers.html#core-controllers)
 * to customize this controller
 */


async function recursiveCommentSearch(id, user, res) {
  const commentChildren = await strapi.query('comment').find({
    parent: id,
    user,
  });
  for (let i = 0; i < commentChildren.length; i += 1) {
    await recursiveCommentSearch(commentChildren[i].id, user, res);
  }
  res.push(id);
}

function getImageUrl(thumb, photo) {
  if (thumb) {
    return {
      url: thumb.url,
    };
  }
  if (photo) {
    return {
      url: photo.url,
    };
  }
  return null;
}

module.exports = {
  getCommentsById: async (ctx) => {
    const {
      postid
    } = ctx.request.header;
    if (postid !== undefined) {
      const id = ctx.state.user.id;
      const comments = await strapi.query('comment').find({
        post: postid
      }, ['user', 'user.photo', 'user.thumb', 'user.admin']);
      let filteredComments = [];
      for (let i = 0; i < comments.length; i++) {
        if (comments[i].user) {
          const banned = await strapi.services.ban.checkBan(id, comments[i].user.id);
          const muted = await strapi.services.commentban.checkBan(id, comments[i].id);
          if ((!banned) && (!muted)) {
            filteredComments.push(comments[i]);
          }
        }
      }
      return ctx.send(filteredComments.map(({
        user: {
          username,
          name,
          photo,
          thumb,
          admin,
          id: user_id,
        },
        ...comment
      }) => ({
        ...comment,
        user: {
          id: user_id,
          username,
          name,
          photo: getImageUrl(thumb, photo),
          me: user_id === id,
          admin
        }
      })));
    } else {
      return ctx.send({
        statusCode: 402,
      });
    }
  },
  getCommentsSingle: async (ctx) => {
    const id = ctx.state.user.id;
    const {
      postid
    } = ctx.request.header;
    if (postid !== undefined) {
      const comments = await strapi.query('comment').find({
        post: postid
      }, ['user', 'user.photo', 'user.thumb', 'user.admin']);
      let filteredComments = [];
      for (let i = 0; i < comments.length; i++) {
        if (comments[i].user) {
          const banned = await strapi.services.ban.checkBan(id, comments[i].user.id);
          const muted = await strapi.services.commentban.checkBan(id, comments[i].id);
          if ((!banned) && (!muted) && (!comments[i].parent)) {
            filteredComments.push(comments[i]);
          }
        }
      }
      const comment = filteredComments[filteredComments.length - 1];
      if (comment) {
        const count = await strapi.query('comment').count({
          post: postid
        });
        const {
          photo,
          thumb,
          username,
          name,
          admin
        } = comment.user;
        return ctx.send({
          comment: {
            ...comment,
            user: {
              id: comment.user.id,
              username,
              name,
              admin,
              photo: getImageUrl(thumb, photo),
              me: (comment.user && comment.user.id) === id ? true : undefined,
            }
          },
          count,
        });

      } else {
        return ctx.send({
          comment: null,
          count: 0,
        });
      }
    } else {
      return ctx.send({
        statusCode: 402,
      });
    }
  },
  createComment: async (ctx) => {
    const id = ctx.state.user.id;
    const {
      comment,
      parent,
      post,
    } = ctx.request.body;
    if (post) {
      return ctx.send(await strapi.query('comment').create({
        comment,
        parent: parent ? parent : undefined,
        post,
        user: id,
      }));
    } else {
      return ctx.send({
        statusCode: 402,
      });
    }
  },
  deleteComment: async (ctx) => {
    const user = ctx.state.user.id;
    const {
      id,
    } = ctx.request.body;
    const res = new Array(0);
    await recursiveCommentSearch(id, user, res);
    for (let i = 0; i < res.length; i += 1) {
      res[i];
      await strapi.query('comment').delete({
        id: res[i],
        user,
      });
    }
    return ctx.send({
      statusCode: 200,
    });
  }
};

