'use strict';

/**
 * Read the documentation () to implement custom controller functions
 */

async function handleEmotion(id, post) {
   let query = await strapi.query('emotion').findOne({
      post,
   }, []);

   if (!query) {
      query = await strapi.query('emotion').create({
         post,
         reactions: [{
            user: id,
            reaction: -1,
         }]
      })
   }
   return query
}

module.exports = {
   createPostEmotion: async (ctx) => {
      const id = ctx.state.user.id;
      const {
         post,
         emotion,
      } = ctx.request.body
      if (!id) {
         return ctx.notFound('No user 404');
      }
      if (!post) {
         return ctx.notFound('No post id 404');
      }
      if (!emotion) {
         return ctx.notFound('No emotion specified 404');
      }
      const query = await handleEmotion(id, post)
      let reactions = []
      if(typeof query.reactions === "string") {
         reactions = JSON.parse(query.reactions)
      } else {
         reactions = query.reactions
      }
      if (Array.isArray(reactions)) {
         const index = reactions.findIndex(reaction => reaction.user === id)
         if (index !== -1) {
            reactions[index].reaction = emotion;
         } else {
            reactions.push({
               user: id,
               reaction: emotion,
            })
         }
      }
      return ctx.send(await strapi.query('emotion').update({
         id: query.id,
      }, {
         reactions,
      }, []));
   },
   getPostEmotions: async (ctx) => {
      const id = ctx.state.user.id
      const {
         post,
         increment,
      } = ctx.request.header;
      if (!id) {
         return ctx.notFound('No user 404');
      }
      if (!post) {
         return ctx.notFound('No post id 404');
      }
      const query = await handleEmotion(id, post)
      if (increment == 1) {
         strapi.query('emotion').update({
            id: query.id,
         }, {
            viewCount: query.viewCount + 1,
         });
      }
      let reactions = []
      if(typeof query.reactions === "string") {
         reactions = JSON.parse(query.reactions)
      } else {
         reactions = query.reactions
      }
      let HAPPY_FACE = 0;
      let CLAP = 0;
      let TOMATO = 0;
      if (Array.isArray(reactions)) {
         reactions.forEach(reaction => {
            switch (reaction.reaction) {
               case 0: case "0":
                  HAPPY_FACE += 1;
                  break;
               case 1: case "1":
                  CLAP += 1;
                  break;
               case 2: case "2":
                  TOMATO += 1;
                  break;
               default:
                  break;
            }
         })
      }
      return ctx.send({
         HAPPY_FACE,
         CLAP,
         TOMATO,
         viewCount: (increment == 1) ? query.viewCount + 1 : query.viewCount,
      });
   },
};
