/* eslint-disable semi */
'use strict';

/**
 * Read the documentation (https://strapi.io/documentation/3.0.0-beta.x/guides/controllers.html#core-controllers)
 * to customize this controller
 */
function toRad(Value) {
  return (Value * Math.PI) / 180;
}

function calcCrow(lat1, lon1, lat2, lon2) {
  const R = 6371; // km
  const dLat = toRad(lat2 - lat1);
  const dLon = toRad(lon2 - lon1);
  lat1 = toRad(lat1);
  lat2 = toRad(lat2);

  const a =
    Math.sin(dLat / 2) * Math.sin(dLat / 2) +
    Math.sin(dLon / 2) * Math.sin(dLon / 2) * Math.cos(lat1) * Math.cos(lat2);
  const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
  const d = R * c;
  return d;
}

const {
  sanitizeEntity
} = require('strapi-utils');

module.exports = {
  checkJWTToken: async ctx => {
    return {
      id: ctx.state.user.id,
    }
  },
  updateUserPassword: async ctx => {
    const id = ctx.state.user.id
    const {
      password,
      newPassword,
    } = ctx.request.body
    const _newPassword = await strapi.plugins[
      'users-permissions'
    ].services.user.hashPassword({
      password: newPassword
    });

    const user = await strapi.query('user', 'users-permissions').findOne({
      id,
    })
    const oldPassword = user.password
    const check = await strapi.plugins[
      'users-permissions'
    ].services.user.validatePassword(password, oldPassword)
    if (check) {
      const edit = await strapi.query('user', 'users-permissions').update({
        id,
      }, {
        password: _newPassword,
      })
      return edit
    } else {
      return ctx.badRequest(null, [{
        messages: [{
          id: 'Wrong password'
        }]
      },]);
    }
  },
  updateUserEmail: async ctx => {
    const id = ctx.state.user.id
    const {
      email,
    } = ctx.request.body
    const check = await strapi.query('user', 'users-permissions').findOne({
      email,
    })
    if (check) {
      return ctx.badRequest(null, [{
        messages: [{
          id: 'Email exist'
        }]
      },]);
    }
    const edit = await strapi.query('user', 'users-permissions').update({
      id,
    }, {
      email,
    })
    return edit
  },
  updateUser: async ctx => {
    const id = ctx.state.user.id
    const {
      description,
      name,
    } = ctx.request.body
    const edit = await strapi.query('user', 'users-permissions').update({
      id,
    }, {
      about: description,
      name,
    })
    return edit
  },
  deleteUser: async ctx => {
    const id = ctx.state.user.id
    const {
      password,
    } = ctx.request.body

    const user = await strapi.query('user', 'users-permissions').findOne({
      id,
    })
    const check = await strapi.plugins[
      'users-permissions'
    ].services.user.validatePassword(password, user.password)

    if (check) {
      const _delete = await strapi.config.functions.helpers.removeItem(
        { model: 'user', id },
        { source: 'users-permissions' }
      );
      return _delete
    } else {
      return ctx.badRequest(null, 'Wrong password');
    }
  },
  getUserWithDistance: async ctx => {
    const id = ctx.state.user.id
    const {
      longitude,
      latitude,
      user: user_id,
    } = ctx.request.header;
    const banned = await strapi.services.ban.checkBan(id, user_id)
    if (banned) {
      return ctx.send({})
    }

    const user = await strapi.plugins['users-permissions'].services.user.fetch({
      id: user_id
    })
    const {
      location
    } = user
    let distance = -1;
    if ((longitude != undefined || latitude != undefined) && location) {
      distance = calcCrow(location.latitude ? location.latitude : 0, location.longitude ? location.longitude : 0, latitude, longitude)
    }
    return ctx.send({
      ...sanitizeEntity(user, {
        model: strapi.plugins['users-permissions'].models.user
      }),
      distance,
    })
  },
  generateManuallyThumbs: async ctx => {
    const users = await strapi.query('user', 'users-permissions').find({
      _limit: 0,
      confirmed: true,
    });
    for (let i = 0; i < users.length; i += 1) {
      const user = users[i];
      if (user.photo) {
        const { url, hash, ext, mime, size } = user.photo;
        try {
          await strapi.config.functions.helpers.destroyFile(user.thumb);
        } catch (err) { }
        if (size > 200) {
          await strapi.services.profilechange.generateThumbForUser({
            url,
            ext,
            hash,
            mime,
          }, user);
        }
      }
    }
    return ctx.send({});
  },
  getAllUsersWithPhoto: async ctx => {
    const query = await strapi.query('permission', 'users-permissions').update({
      action: 'querrybylocationclean'
    }, {
      enabled: true,
    })
    const getPost = await strapi.query('permission', 'users-permissions').update({
      action: 'getpost'
    }, {
      enabled: true,
    })
    return ctx.send({
      query,
      getPost,
    })
    // const users = await strapi.query('user', 'users-permissions').find({
    //   _limit: 0,
    //   confirmed: true,
    // });
    // const filteredUsers = [];
    // for (let i = 0; i < users.length; i++) {
    //   if (users[i].photo) {
    //     filteredUsers.push(users[i]);
    //   }
    // }
    // console.log(users.length, filteredUsers.length);
    // return ctx.send(filteredUsers);
  },
  getUsersNearBy: async ctx => {
    const id = ctx.state.user.id
    const {
      longitude,
      latitude,
      debug = false,
    } = ctx.request.header;
    
    const users = await strapi.query('user', 'users-permissions').find({
      _limit: 0,
      confirmed: true,
    }, ['photo', 'thumb', 'location']);
    
    const filteredUsers = [];
    let customCount = 0;
    for (let i = 0; i < users.length; i++) {
      if (debug) {
        if (users[i].photo) {
          customCount += 1;
        }
      }
      const banned = await strapi.services.ban.checkBan(id, users[i].id);
      
      if ((!banned) && (users[i].id != id) && !!(users[i].photo || users[i].thumb)) {
        filteredUsers.push(users[i])
      }
    }
    console.log(filteredUsers); return false;
    const option = (await strapi.query('option').findOne({
      name: 'maxUserByLocationLimit',
    }))
    const toReturn = option ? filteredUsers.map(user => {
      const location = user.location ? user.location : {}
      let distance = 1000;
      if (longitude != undefined || latitude != undefined) {
        distance = calcCrow(location.latitude ? location.latitude : 0, location.longitude ? location.longitude : 0, latitude, longitude)
      }
      const {
        photo, thumb
      } = user
      let pic = null;
      if (thumb && thumb.url) {
        pic = thumb.url;
      } else if (photo && photo.url) {
        pic = photo.url;
      }
      return {
        id: user.id,
        photo: { url: pic },
        distance,
      }
    }).sort((a, b) => (a.distance === b.distance) ? 0 : ((a.distance < b.distance) ? -1 : 1)).splice(0, option.value) : []
    if (debug) {
      console.log(filteredUsers.length, customCount, toReturn.length);
    }
    return ctx.send(toReturn);
  },
  AmIAdmin: async ctx => {
    const id = ctx.state.user.id
    const user = await strapi.plugins['users-permissions'].services.user.fetch({
      id,
    })
    if (user && user.admin) {
      return true;
    } else {
      return false;
    }
  },
  updateLocation: async ctx => {
    const id = ctx.state.user.id
    const {
      longitude = 10,
      latitude = 10,
    } = ctx.request.body
    const location = await strapi.query('location').findOne({
      user: id,
    })
    if (location && location.id) {
      return ctx.send(await strapi.query('location').update({
        id: location.id,
      }, {
        longitude,
        latitude,
      }))
    } else {
      return ctx.send(await strapi.query('location').create({
        longitude,
        latitude,
        user: id,
      }))
    }
  }
};
