/* eslint-disable quotes */
/* eslint-disable semi */
'use strict';

/**
 * Read the documentation () to implement custom controller functions
 */
const {
  sanitizeEntity
} = require('strapi-utils');

function uuid() {
  var dt = new Date().getTime();
  var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
    var r = (dt + Math.random() * 16) % 16 | 0;
    dt = Math.floor(dt / 16);
    return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
  });
  return uuid;
}
module.exports = {
  composeMessage: async ctx => {
    const id = ctx.state.user.id
    const {
      message,
      recipient,
    } = ctx.request.body
    const _message = await strapi.query('message').findOne({
      sender: [id, recipient],
      receiver: [id, recipient],
    })
    if (_message) {
      const messagesToReset = await strapi.query('message').find({
        head: _message.head,
      })
      for (let i = 0; i < messagesToReset.length; i++) {
        await strapi.query('message').update({
          id: messagesToReset[i].id,
        }, {
          current: false,
        })
      }
      const toReturn = await strapi.query('message').create({
        sender: id,
        receiver: recipient,
        head: _message.head,
        seen: false,
        current: true,
        message,
      })
      strapi.io.emit(recipient, 1);
      strapi.io.emit(_message.head, 1);
      return sanitizeEntity(toReturn, {
        model: strapi.models.message
      })
    } else {
      const toReturn = await strapi.query('message').create({
        sender: id,
        receiver: recipient,
        head: uuid(),
        seen: false,
        current: true,
        message,
      })
      strapi.io.emit(recipient, 1);
      strapi.io.emit(toReturn.head, 1);
      return sanitizeEntity(toReturn, {
        model: strapi.models.message
      })
    }
  },
  getUnreadMessagesCount: async ctx => {
    const id = ctx.state.user.id
    const messages = await strapi.query('message').find({
      receiver: id,
      seen: false,
    }, [])
    return messages.length
  },
  getMessagesByHead: async ctx => {
    const id = ctx.state.user.id
    const {
      user,
    } = ctx.request.header
    // if (head != 0) {

    //   const sender = await strapi.query('message').find({
    //     sender: id,
    //     head,
    //   })
    //   const receiver = await strapi.query('message').find({
    //     receiver: id,
    //     head,
    //   })
    //   return [...sender, ...receiver, ]
    // } else
    if (user) {

      const banned = await strapi.services.ban.checkBan(id, user)
      if (banned) return ctx.send([])
      const messages = await strapi.query('message').find({
        sender: [id, user],
        receiver: [id, user],
      }, [])
      for (let i = 0; i < messages.length; i++) {
        if (messages[i].receiver == id) {
          await strapi.query('message').update({
            id: messages[i].id,
          }, {
            seen: true,
          })
        }
      }
      return messages.reverse()
    } else {
      return ctx.badRequest(null, [{
        messages: [{
          id: 'No recipient'
        }]
      }, ]);
    }
  },
  getMessagesHeads: async ctx => {
    const id = ctx.state.user.id
    const sender = await strapi.query('message').find({
      sender: id,
      current: true,
      _sort: 'createdAt:desc',
    }, ['receiver', 'receiver.photo', 'sender', 'sender.photo', ])
    const receiver = await strapi.query('message').find({
      receiver: id,
      current: true,
      _sort: 'createdAt:desc',
    }, ['receiver', 'receiver.photo', 'sender', 'sender.photo', ])
    const messages = [...sender, ...receiver, ];
    let messagesFiltered = []
    for (let i = 0; i < messages.length; i++) {
      try {
        const banned = await strapi.services.ban.checkBan(messages[i].receiver.id, messages[i].sender.id)
        if (!banned) {
          messagesFiltered.push(messages[i])
        }
      } catch(e) { console.log('error'); }
    }
    messagesFiltered.sort((a,b) => { return new Date(b.createdAt) - new Date(a.createdAt)});
    return messagesFiltered
  },
  remove: async (ctx) => {
    const id = ctx.state.user.id;
    const {
      id: message_id
    } = ctx.request.body;
    const message = await strapi.query('message').findOne({
      id: message_id,
    }, []);
    if (message.sender == id && !message.message) {
      return ctx.send(await strapi.query('message').delete({
        id: message.id,
      }));
    }
    return ctx.badRequest(null, [{
      messages: [{
        id: 'Not the same user'
      }]
    },]);

  },
};
