/* eslint-disable no-empty */
/* eslint-disable quotes */
'use strict';

/**
 * Read the documentation (https://strapi.io/documentation/3.0.0-beta.x/guides/controllers.html#core-controllers)
 * to customize this controller
 */
function toRad(Value) {
  return (Value * Math.PI) / 180;
}

function calcCrow(lat1, lon1, lat2, lon2) {
  const R = 6371; // km
  const dLat = toRad(lat2 - lat1);
  const dLon = toRad(lon2 - lon1);
  lat1 = toRad(lat1);
  lat2 = toRad(lat2);

  const a =
    Math.sin(dLat / 2) * Math.sin(dLat / 2) +
    Math.sin(dLon / 2) * Math.sin(dLon / 2) * Math.cos(lat1) * Math.cos(lat2);
  const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
  const d = R * c;
  return d;
}

function getImageUrl(thumb, photo) {
  if (thumb) {
    return {
      url: thumb.url,
    };
  }
  if (photo) {
    return {
      url: photo.url,
    };
  }
  return null;
}

module.exports = {
  getPost: async ctx => {
    const {
      post: id,
    } = ctx.request.header;
    const userId = ctx.state.user.id;
    const post = await strapi.query('post').findOne({
      _id: id,
    }, ["user", "user.photo", "user.thumb", "user.admin"]);
    const user = post.user;
    const { photo, username, name, admin, thumb } = post.user || {};

    let commentToSend = null;
    let commentCount = 0;

    const comments = await strapi.query('comment').find({
      post: id
    }, ['user', 'user.photo', 'user.thumb', 'user.admin']);
    let filteredComments = [];
    for (let i = 0; i < comments.length; i++) {
      if (comments[i].user) {
        const banned = await strapi.services.ban.checkBan(userId, comments[i].user.id);
        const muted = await strapi.services.commentban.checkBan(userId, comments[i].id);
        if ((!banned) && (!muted) && (!comments[i].parent)) {
          filteredComments.push(comments[i]);
          commentCount += 1;
        }
      }
    }
    const comment = filteredComments[filteredComments.length - 1];
    if (comment) {
      const {
        photo,
        thumb,
        username,
        name,
        admin
      } = comment.user;
      commentToSend = {
        ...comment,
        user: {
          id: comment.user.id,
          username,
          name,
          admin,
          photo: getImageUrl(thumb, photo),
          me: (comment.user && comment.user.id) === id ? true : undefined,
        }
      };
    }
    return ctx.send({
      ...post,
      user: user ? {
        id: user.id,
        admin,
        photo: getImageUrl(thumb, photo), username, name,
        me: ((user && user.id) === id) ? true : undefined,
      } : {},
      comment: commentToSend,
      count: commentCount,
    });
  },
  querryByLocationClean: async ctx => {
    const id = ctx.state.user.id;
    const {
      longitude,
      latitude
    } = ctx.request.header;
    const query = await strapi.query('post').find({
      _limit: 0,
    }, ["user"]);


    let postFiltered = [];
    for (let i = 0; i < query.length; i++) {
      const post = query[i];
      if ((!post.vip) && post.user) {
        if (post.approved || (id === (post.user.id))) {
          const banned = await strapi.services.ban.checkBan(id, post.user.id);
          const muted = await strapi.services.postban.checkBan(id, post.id);
          if (!banned && !muted) {
            postFiltered.push(post);
          }
        }
      }
    }

    const toSend = postFiltered.map(post => {
      let distance = 10;
      if (longitude != undefined || latitude != undefined) {
        distance = calcCrow(post.latitude, post.longitude, latitude, longitude);
      } else if (id == post.user.id) {
        distance = 0;
      }
      return {
        id: post.id,
        distance,
      };
    }).sort((a, b) => (a.distance === b.distance) ? 0 : ((a.distance < b.distance) ? -1 : 1));
    const vip = query.filter(post => post.vip)
      .map(post => ({
        id: post.id,
      }));
    const option = (await strapi.query('option').findOne({
      name: "maxPostByLocationLimit",
    }));
    return ctx.send(option ? [...vip, ...toSend].splice(0, option.value) : []);
  },
  querryByLocation: async ctx => {
    const id = ctx.state.user.id;
    const {
      longitude,
      latitude
    } = ctx.request.header;
    const query = await strapi.query('post').find({
      _limit: 0,
    }, ["user", "user.photo", "user.thumb", "user.admin"]);


    let postFiltered = [];
    for (let i = 0; i < query.length; i++) {
      const post = query[i];
      if ((!post.vip) && post.user) {
        if (post.approved || (id === (post.user.id))) {
          const banned = await strapi.services.ban.checkBan(id, post.user.id);
          const muted = await strapi.services.postban.checkBan(id, post.id);
          if (!banned && !muted) {
            postFiltered.push(post);
          }
        }
      }
    }


    const toSend = postFiltered.map(post => {
      let distance = 10;
      if (longitude != undefined || latitude != undefined) {
        distance = calcCrow(post.latitude, post.longitude, latitude, longitude);
      }  if (id === post.user.id) {
        distance = 0;
      }
      const user = post.user;
      const { photo, username, name, thumb, admin } = post.user || {};
      return {
        ...post,
        user: user ? {
          id: user.id,
          admin,
          photo: getImageUrl(thumb, photo), username, name,
          me: ((user && user.id) === id) ? true : undefined,
        } : {},
        distance,
      };
    }).sort((a, b) => (a.distance === b.distance) ? 0 : ((a.distance < b.distance) ? -1 : 1));
    const vip = query.filter(post => post.vip)
      .map(post => ({
        ...post,
        user: {
          ...post.user,
          me: ((post.user && post.id) === id) ? true : undefined,
        } || {}
      }));
    const option = (await strapi.query('option').findOne({
      name: "maxPostByLocationLimit",
    }));
    return ctx.send(option ? [...vip, ...toSend].splice(0, option.value) : []);
  },
  removePost: async (ctx) => {
    const {
      id: post_id
    } = ctx.request.body;

    const post = await strapi.query('post').findOne({
      id: post_id,
    }, []);
    try {
      try {
        const m = post.media[0];
        if (m) {
          await strapi.config.functions.helpers.destroyFile(m);
        }
      } catch (error) { }
      return ctx.send(await strapi.query('post').delete({
        id: post.id,
      }));
    } catch (e) {
      return ctx.badRequest(null, [{
        messages: [{
          id: 'Not the same user'
        }]
      },]);
    }

  },
  getUnapprovedPosts: async (ctx) => {
    const posts = await strapi.query('post').find({
      _limit: 0,
      approved: false,
    });
    return ctx.send(posts);
  },
  createPost: async (ctx) => {
    const id = ctx.state.user.id;    
    const {
      about,
      longitude = 10,
      latitude = 10,
    } = ctx.request.body;
    let { vip } = ctx.request.body;
    //defaultPostLimit
    //should check post limit for user
    //now app calls canPost,checks if can post and then call this
    //technically user can post outside app as many times
    if (about) {

      if (vip) {
        const user = await strapi.plugins['users-permissions'].services.user.fetch({
          id,
        });
        if (!(user && user.admin)) {
          vip = false;
        }
      }

      /********** Brijesh ********/
      // const querys = await strapi.query('post').find({
      //   _limit: 0,
      // }, ["user"]);


      // let postIdFiltered = [];
      // for (let i = 0; i < querys.length; i++) {
      //   const post = querys[i];
      //   let postId = post._id;
      //   // console.log(postId);
      //   postIdFiltered.push(postId);
      // }
      //const media = await strapi.query('file', 'upload').find({"related.ref":'5eb46fdd0df4397c76d2b1fb',_limit: 1});
      // const media = await strapi.query('file', 'upload').find({_limit: 0});
      
      //let fileData = [];
     

      // for (let m = 0; m < media.length; m++) {
          
      //     var cType = media[m].related[0].__contentType;
      //     var postId = media[m].related[0].id;
       
      //   if(cType == 'Post'){
      //     if(postId !== undefined){
      //       var mediaId = media[m].id;
      //       console.log(postId);
      //     }
      //   }
      // } return false;
      // media.forEach(async function (items) {
      //   if(items.related[0] !== undefined){ 
      //     var cType = items.related[0].__contentType;
      //     var postId = items.related[0].id;
           
      //     if(cType == 'Post'){
      //       if(postId !== undefined){

      //           try {
      //             var mediaId  = items.id;
                  
      //             //var newMedia = '[{'+ mediaId.toString() +'}]';                
      //             await strapi.query('post').update({
      //               _id: postId.toString(),
      //             }, {
      //               media:  mediaId.toString(),
      //             });                
      //             console.log(mediaId);
      //           } catch (error) { console.log(error); }   
      //       }
      //     }
      //   }
      // });
      // console.log('test');
      // return false;
      // for (let i = 0; i < media.length; i++) {
      //   const med   = media.related;
      //   var related = media[i].related[0];
      //   console.log(med);
        // if (med.length > 0) {
        //   var newvalues = { $set: {} };
        //   newvalues.$set[related.field] = media._id;

        // }
        
        // for (let j = 0; j < related.length; j++) {
        //     const rel = related[i];
        //     const media = await strapi.query('file', 'upload').find({
        //       _limit: 0,
        //     }, ["related"]); 
          
        // } return false;
        // let postId = post._id;
        // fileData.push(postId);
     //}
      // const upload_avatar  = await strapi.query('upload_file').find({
      //   'related.ref': ObjectId('617727adb520d8415fca40ab'),
      // });


      
      //return false;
      /********* End ****/
      const post = await strapi.query('post').create({
        about,
        longitude,
        latitude,
        user: id,
        approved: false,
        vip,
      });

      // should send an email
      strapi.io.emit("alert_admin_new_post", 1);
      return ctx.send(post);
    } else {
      return ctx.send({
        statusCode: 402,
      });
    }
  },
  canPost: async (ctx) => {
    const user = ctx.state.user.id;
    const currentTime = new Date();
    currentTime.setHours(0, 0, 0, 0);
    if (!user) return ctx.send({
      status: false,
    });
    const posts = await strapi.query('post').find({
      user,
    });

    const { value: limit } = (await strapi.query('option').findOne({
      name: "defaultPostLimit",
    }));

    const { customLimit } = (await strapi.query('user', 'users-permissions').findOne({
      id: user,
    }));
    if (customLimit == undefined) {
      if ((limit <= 0)) return ctx.send({
        status: false,
      });
    } else {
      if ((customLimit <= 0)) return ctx.send({
        status: false,
      });
    }
    const currentPostCount = posts.filter(post => currentTime < post.created_at).length;
    return ctx.send({
      status: customLimit == undefined ? (currentPostCount <= limit) : (currentPostCount <= customLimit),
    });

  },
  approvePost: async (ctx) => {
    const { id } = ctx.params;
    return ctx.send(await strapi.query('post').update({
      id,
    }, {
      approved: true,
    }));
  }, 
  imageMigrate: async(ctx) => {
    const result = await strapi.query('post').find();
    console.log(result); return false;
    return ctx.send(result ? result : []);   
  },
};
