/* eslint-disable no-empty */
'use strict';

/**
 * Read the documentation () to implement custom controller functions
 */

const {
  sanitizeEntity
} = require('strapi-utils');
module.exports = {
  approve: async ctx => {
    const { id } = ctx.params;
    const profileChanges = await strapi.query('profilechange').findOne({
      user: id,
    }, ['user', 'user.photo']);
    // const oldProfile = await strapi.query('user', 'users-permissions').findOne({
    //   id,
    // }, ['user', 'user.photo']);
    // const oldProfileId = oldProfile.photo.id;

    const config = await strapi.store({
      environment: strapi.config.environment,
      type: 'plugin',
      name: 'upload',
    }).get({ key: 'provider' });
    //console.log(profileChanges.user.photo.id); return false;
    if (profileChanges && profileChanges.photo) {
      const { url, hash, ext, mime, size } = profileChanges.photo;

      // {
      //   _id: 5ebe9efa6703d33ca44130b0,
      //   name: 'Hirsutism (2).png',
      //   sha256: 'GJGwuTU3OuRGx_tDV+IhxGB51YgVMfp6vn3AYKsJ5Mk',
      //   hash: 'e79c0b44b0764f0db8fc4f7883e8da91',
      //   ext: '.png',
      //   mime: 'image/png',
      //   size: '206.96',
      //   url: '/uploads/e79c0b44b0764f0db8fc4f7883e8da91.png',
      //   provider: 'local',
      //   related: [ 5ebe9ef96703d33ca44130af ],
      //   createdAt: 2020-05-15T13:54:02.473Z,
      //   updatedAt: 2020-05-15T13:54:02.626Z,
      //   __v: 0,
      //   id: '5ebe9efa6703d33ca44130b0'
      // }
      
   
      
      // const deletedOld = await strapi.query('file', 'upload').delete({
      //   _id: '61fccbebac6dc02b148b7ac4',
      // });

      await strapi.query('user', 'users-permissions').update({
        id: profileChanges.user.id,
      }, {
        approvedChange: true,
        photo:profileChanges.photo.id,
        thumb:profileChanges.photo.id,
      });
      await strapi.query('file', 'upload').update({
        id: profileChanges.photo.id,
      }, {
        'related.0.refId': profileChanges.user.id,
        'related.0.ref': profileChanges.user.id,
        'related.0.kind': 'UsersPermissionsUser',
      });
      // if(size > 200) {
      //   await strapi.services.profilechange.generateThumbForUser({
      //     url,
      //     ext,
      //     hash,
      //     mime,
      //   }, profileChanges.user);
      // }
      const deletedChange = await strapi.query('profilechange').delete({
        id: profileChanges.id,
      });
      return deletedChange;
    } else {
      return ctx.badRequest(null, [
        {
          message: 'Missing file!',
        },
      ]);
    }
  },
  url: async ctx => {
    const user = ctx.state.user.id;
    const changes = await strapi.query('profilechange').findOne({
      user,
    });
    if(changes) {
      const {photo} = changes;
      if(photo && photo.url) {
        return {url: photo.url};
      }
    }
    throw '';
  },
  new: async ctx => {
    const user = ctx.state.user.id;
    const oldChanges = await strapi.query('profilechange').find({
      user,
    });
    if(oldChanges) {
      for(let i = 0; i < oldChanges.length; i += 1) {
        const item = oldChanges[i];
        try {
          await strapi.config.functions.helpers.destroyFile(item.photo);
        } catch (error) {}
        await strapi.query('profilechange').delete({
          id: item.id,
        });
      }
    }
    const newChange = await strapi.query('profilechange').create({
      user
    });
    return ctx.send(sanitizeEntity(newChange, {
      model: strapi.models.profilechange
    }));
  },
  finalize: async ctx => {
    const user = ctx.state.user.id;
    await strapi.query('user', 'users-permissions').update({
      id: user,
    }, {
      approvedChange: false,
    });
    return ctx.send(true);
  },
  remove: async (ctx) => {
    const {
      id: new_change_id,
    } = ctx.request.body;

    try {
      if(!new_change_id) {
        throw new Error('no Id');
      }
      const oldItem = await strapi.query('profilechange').findOne({
        id: new_change_id,
      });
      try {
        await strapi.config.functions.helpers.destroyFile(oldItem.photo);
      } catch (e) { }
      const changesCount = await strapi.query('profilechange').count({
        user: oldItem.user,
      });
      if(changesCount <= 1) {
        await strapi.query('user', 'users-permissions').update({
          id: oldItem.user,
        }, {
          approvedChange: true,
        });
      }
      return ctx.send(await strapi.query('profilechange').delete({
        id: new_change_id,
      }));
    } catch (e) {
      console.log(e);
      return ctx.badRequest(null, [{
        messages: [{
          id: 'Not the same user'
        }]
      },]);
    }
  },
};
