'use strict';

/**
 * Read the documentation () to implement custom service functions
 */

const sharp = require('sharp');

const crypto = require('crypto');
const uuid = require('uuid/v4');

function niceHash(buffer) {
  return crypto
    .createHash('sha256')
    .update(buffer)
    .digest('base64')
    .replace(/=/g, '')
    .replace(/\//g, '-')
    .replace(/\+/, '_');
}

module.exports = {
  removeAll: async (user) => {
    const changes = await strapi.query('profilechange').find({
      user,
    }, []);
    for (let i = 0; i < changes.length; i++) {
      const m = changes[i].photo;
      if (m) {
        await strapi.config.functions.helpers.destroyFile(m);
      }
    }
    await strapi.query('profilechange').delete({
      user,
    });
    return true;
  },
  generateThumbForUser: async ({
    url,
    ext,
    hash,
    mime,
  }, user) => {
    try {
      const thumbnailBuffer = await sharp(`public${url}`).rotate().resize({
        width: 256,
        height: 256,
        fit: 'cover',

      }).toBuffer();
      const config = await strapi
        .store({
          environment: strapi.config.environment,
          type: 'plugin',
          name: 'upload',
        })
        .get({ key: 'provider' });
      const thumbnail = {
        name: `thumb_${hash}${ext}`,
        sha256: niceHash(thumbnailBuffer),
        hash: uuid().replace(/-/g, ''),
        ext,
        buffer: thumbnailBuffer,
        mime,
        size: (thumbnailBuffer.length / 1000).toFixed(2),
      };
      Object.assign(thumbnail, {
        related: [
          {
            refId: user.id,
            ref: 'user',
            source: 'users-permissions',
            field: 'thumb',
          },
        ],
      });
      Object.assign(thumbnail, {
        path: 'thumbs',
      });
      return await strapi.plugins['upload'].services.upload.upload([thumbnail], config);
    } catch (err) {
      console.error(err);
    }
  }
};
